package com.example;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import java.util.Set;
import java.util.Vector;


// 1. must read a large file
// 2. construct an index structure representing all the lines
// 3. enable us to find a particular line (in our app, this will be done a lot!)
// 4. reverse every other line
// 5. print some reversed lines
// 6. instrument some profiling so you know what aspects of the application take the most time
public class RefactoredParser {
	private String text = "bible.txt";
	private LinkedHashMap map = new LinkedHashMap<String, String>();
	
	public static void main(String[] args) {
		Long start = System.currentTimeMillis();
	    System.out.println("Time: 0 milliseconds");
		RefactoredParser parser = new RefactoredParser();
		try {
			parser.buildStructure();
		} catch (Exception e) {
			e.printStackTrace();
		}
	    System.out.println("Time: "+(System.currentTimeMillis()-start) + " milliseconds");
		
		// find the reference (ignore case)
		parser.findReference("Matthew 1:1");
		parser.findReference("1 Timothy 4:3");
	    System.out.println("Time: "+(System.currentTimeMillis()-start) + " milliseconds");
		    
        // print a few lines that should be reversed
	    parser.getByIndex(0);
	    parser.getByIndex(2);
	    System.out.println("Time: "+(System.currentTimeMillis()-start) + " milliseconds");
	}
	
	private void buildStructure() throws Exception {
		// read the file
		Scanner reader = new Scanner(new File(text));
		
		boolean reverse = true;
		String verse;
		while( reader.hasNextLine() ) {
			verse = reader.nextLine();
//			System.out.println("Reading line: "+verse + " milliseconds");

			String[] aLine = verse.split("\t");
			if (reverse) map.put(aLine[0], reverseLine(verse));
			else map.put(aLine[0], verse);
			
			reverse = !reverse;
//			System.out.println("Adding line: " + verse + " milliseconds");
		}
	}
	
	private String reverseLine(String line) {
		return (new StringBuilder(line).reverse().toString());
	}
	
	private void findReference(String reference) {
		String verse = (String) map.get(reference);
		if (null != verse)
			System.out.println("Found line: "+ verse);
		else System.out.println("Line not found.");
	}

	private void getByIndex(int index) {
		List<List<String>> l = new ArrayList<List<String>>(map.values());
		System.out.println("By index: "+l.get(index));
	}
	
}
